"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var apollo_server_core_1 = require("apollo-server-core");
var gql_1 = require("./gql");
function graphqlLodashExpress(options) {
    if (!options) {
        throw new Error('Apollo Server requires options.');
    }
    if (arguments.length > 1) {
        throw new Error("Apollo Server expects exactly one argument, got " + arguments.length);
    }
    return function (req, res, next) {
        var originalQuery = req.method === 'POST' ? req.body : req.query;
        var transform, apply, isBatch = false;
        if (Array.isArray(originalQuery)) {
            isBatch = true;
            var res_1 = originalQuery.map(function (q) { return gql_1.graphqlLodash(q.query, q.operationName); });
            transform = res_1.map(function (t) { return t.transform; });
            apply = res_1.map(function (t) { return t.apply; });
        }
        else {
            var res_2 = gql_1.graphqlLodash(originalQuery.query, originalQuery.operationName);
            transform = res_2.transform;
            apply = res_2.apply;
        }
        apollo_server_core_1.runHttpQuery([req, res], {
            method: req.method,
            options: options,
            query: originalQuery,
        })
            .then(function (gqlResponse) {
            if (isBatch) {
                var result = JSON.parse(gqlResponse);
                return JSON.stringify(result.map(function (r, i) {
                    if (apply[i]) {
                        return transform[i](r);
                    }
                    else {
                        return r;
                    }
                }));
            }
            else {
                if (apply) {
                    return JSON.stringify({
                        data: transform(JSON.parse(gqlResponse).data),
                    });
                }
                else {
                    return gqlResponse;
                }
            }
        })
            .then(function (gqlResponse) {
            res.setHeader('Content-Type', 'application/json');
            res.write(gqlResponse);
            res.end();
        }, function (error) {
            if ('HttpQueryError' !== error.name) {
                return next(error);
            }
            if (error.headers) {
                Object.keys(error.headers).forEach(function (header) {
                    res.setHeader(header, error.headers[header]);
                });
            }
            res.statusCode = error.statusCode;
            res.write(error.message);
            res.end();
        });
    };
}
exports.graphqlLodashExpress = graphqlLodashExpress;
//# sourceMappingURL=express.js.map